@@ -8,6 +8,8 @@ from pyensembl import Genome
 from gwaslab.Log import Log
 from gwaslab.download import download_ref
 from gwaslab.download import check_and_download
+from gwaslab.download import update_formatbook
+from gwaslab.config import options
 
 #hard-coded data
 def get_chr_to_NC(build,inverse=False):
@@ -142,43 +144,39 @@ def get_number_to_NC(build,inverse=False):
 def get_NC_to_number(build):
     return get_number_to_NC(build=build,inverse=True)
 
-
-
-def get_chr_list(add_number=False):
-    chrom_list=["1","2","3","4","5","6","7","8","9","10",
-            "11","12","13","14","15","16","17","18",
-            "19","20","21","22","23","24","25","X","Y","M","MT"]
+def get_chr_list(add_number=False,n=25):
+    chrom_list=[str(i) for i in range(1,n+1)]+["X","Y","M","MT"]
+    
     if add_number is True:
-        chrom_list = ["1","2","3","4","5","6","7","8","9","10",
-            "11","12","13","14","15","16","17","18",
-            "19","20","21","22","23","24","25","X","Y","M","MT"] +[i for i in range(1,26)]
+        chrom_list = [str(i) for i in range(1,n+1)] + ["X","Y","M","MT"] + [i for i in range(1,n+1)]
+
     return chrom_list
 
-def get_chr_to_number(out_chr=False,xymt=["X","Y","MT"]):
+def get_chr_to_number(out_chr=False,xymt=["X","Y","MT"],xymt_num=[23,24,25]):
     if out_chr is True:
-        dic= {str(i):str(i) for i in range(1,26)}
-        dic[xymt[0]]="23"
-        dic[xymt[1]]="24"
-        dic[xymt[2]]="25"
+        dic= {str(i):str(i) for i in range(1,200)}
+        dic[xymt[0]]=str(xymt_num[0])
+        dic[xymt[1]]=str(xymt_num[1])
+        dic[xymt[2]]=str(xymt_num[2])
     else:
-        dic= {str(i):i for i in range(1,26)}
-        dic[xymt[0]]=23
-        dic[xymt[1]]=24
-        dic[xymt[2]]=25
-        dic["M"]=25
+        dic= {str(i):i for i in range(1,200)}
+        dic[xymt[0]]= xymt_num[0]
+        dic[xymt[1]]= xymt_num[1]
+        dic[xymt[2]]= xymt_num[2]
+        dic["M"] = xymt_num[2]
     return dic
 
-def get_number_to_chr(in_chr=False,xymt=["X","Y","MT"],prefix=""):
+def get_number_to_chr(in_chr=False,xymt=["X","Y","MT"],xymt_num=[23,24,25],prefix=""):
     if in_chr is True:
-        dic= {str(i):prefix+str(i) for i in range(1,23)}
-        dic["23"]=prefix+xymt[0]
-        dic["24"]=prefix+xymt[1]
-        dic["25"]=prefix+xymt[2]
+        dic= {str(i):prefix+str(i) for i in range(1,200)}
+        dic[str(xymt_num[0])]=prefix+xymt[0]
+        dic[str(xymt_num[1])]=prefix+xymt[1]
+        dic[str(xymt_num[2])]=prefix+xymt[2]
     else:
-        dic= {i:prefix+str(i) for i in range(1,23)}
-        dic[23]=prefix+xymt[0]
-        dic[24]=prefix+xymt[1]
-        dic[25]=prefix+xymt[2]
+        dic= {i:prefix+str(i) for i in range(1,200)}
+        dic[xymt_num[0]]=prefix+xymt[0]
+        dic[xymt_num[1]]=prefix+xymt[1]
+        dic[xymt_num[2]]=prefix+xymt[2]
     return dic
 
 
@@ -192,7 +190,10 @@ def get_high_ld(build="19"):
     return data_path
 
 def get_format_dict(fmt,inverse=False):
-    data_path =  path.dirname(__file__) + '/data/formatbook.json'
+    #data_path =  path.dirname(__file__) + '/data/formatbook.json'
+    data_path = options.paths["formatbook"]
+    if not path.exists(data_path):
+        update_formatbook()
     dicts = json.load(open(data_path))
     dic_meta = dicts[fmt]["meta_data"]
     dic_dict = dicts[fmt]["format_dict"]
@@ -202,7 +203,8 @@ def get_format_dict(fmt,inverse=False):
     return dic_meta, dic_dict
 
 def get_formats_list():
-    data_path =  path.dirname(__file__) + '/data/formatbook.json'
+    #data_path =  path.dirname(__file__) + '/data/formatbook.json'
+    data_path = options.paths["formatbook"]
     dicts = json.load(open(data_path))
     format_list = list(dicts.keys())
     return format_list
@@ -210,32 +212,31 @@ def get_formats_list():
 def get_recombination_rate(chrom, build="19"):
     #http://ftp.1000genomes.ebi.ac.uk/vol1/ftp/technical/working/20110106_recombination_hotspots/
     if build=="19":
-        data_path =  path.dirname(__file__) + '/data/recombination/hg19/genetic_map_GRCh37_chr'+str(chrom)+'.txt.gz'
+        data_path =  options.paths["data_directory"] + 'recombination/hg19/genetic_map_GRCh37_chr'+str(chrom)+'.txt.gz'
         if path.exists(data_path):
             recombination_rate = pd.read_csv(data_path,sep="\t")
         else:
-            if not path.exists(path.dirname(__file__) + '/data/recombination/hg19/'):
-                os.makedirs(path.dirname(__file__) + '/data/recombination/hg19/')
-            download_ref("recombination_hg19",directory = path.dirname(__file__)+'/data/recombination/hg19/')
-            file = tarfile.open(path.dirname(__file__)+'/data/recombination/hg19/recombination_hg19.tar.gz')
-            file.extractall(path.dirname(__file__)+'/data/recombination/hg19/')
+            if not path.exists(options.paths["data_directory"] + 'recombination/hg19/'):
+                os.makedirs(options.paths["data_directory"] + 'recombination/hg19/')
+            download_ref("recombination_hg19",directory = options.paths["data_directory"] + 'recombination/hg19/')
+            file = tarfile.open(options.paths["data_directory"]+'recombination/hg19/recombination_hg19.tar.gz')
+            file.extractall(options.paths["data_directory"]+'recombination/hg19/')
             file.close()
             recombination_rate = pd.read_csv(data_path,sep="\t")
     elif build=="38":
-        data_path =  path.dirname(__file__) + '/data/recombination/hg38/genetic_map_GRCh38_chr'+str(chrom)+'.txt.gz'
+        data_path =  options.paths["data_directory"] + 'recombination/hg38/genetic_map_GRCh38_chr'+str(chrom)+'.txt.gz'
         if path.exists(data_path):
             recombination_rate = pd.read_csv(data_path,sep="\t")
         else:
-            if not path.exists(path.dirname(__file__) + '/data/recombination/hg38/'):
-                os.makedirs(path.dirname(__file__) + '/data/recombination/hg38/')
-            download_ref("recombination_hg38",directory = path.dirname(__file__)+'/data/recombination/hg38/')
-            file = tarfile.open(path.dirname(__file__)+'/data/recombination/hg38/recombination_hg38.tar.gz')
-            file.extractall(path.dirname(__file__)+'/data/recombination/hg38/')
+            if not path.exists( options.paths["data_directory"] + 'recombination/hg38/'):
+                os.makedirs( options.paths["data_directory"] + 'recombination/hg38/')
+            download_ref("recombination_hg38",directory = options.paths["data_directory"]+'recombination/hg38/')
+            file = tarfile.open(options.paths["data_directory"]+'recombination/hg38/recombination_hg38.tar.gz')
+            file.extractall(options.paths["data_directory"]+'recombination/hg38/')
             file.close()
             recombination_rate = pd.read_csv(data_path,sep="\t")
     return recombination_rate
 
-
 ####################################################################################################################
 def get_gtf(chrom, build="19",source="ensembl"):
     if source=="ensembl":
@@ -270,8 +271,8 @@ def get_gtf(chrom, build="19",source="ensembl"):
         
 def gwaslab_info():
     dic={
-   "version":"3.3.5",
-   "release_date":"20221104"
+   "version":"3.3.18",
+   "release_date":"20230108"
     }
     return dic       
         
