@@ -33,6 +33,8 @@ from gwaslab.filtervalue import sampling
 from gwaslab.mqqplot import mqqplot
 from gwaslab.calculate_gc import lambdaGC
 from gwaslab.getsig import getsig
+from gwaslab.getdensity import getsignaldensity
+from gwaslab.getdensity import assigndensity
 from gwaslab.getsig import annogene
 from gwaslab.getsig import getnovel
 from gwaslab.fill import filldata
@@ -45,6 +47,7 @@ from gwaslab.CommonData import get_chr_to_number
 from gwaslab.CommonData import get_high_ld
 from gwaslab.CommonData import get_format_dict
 from gwaslab.CommonData import get_formats_list
+from gwaslab.version import _show_version
 import gc
 
 #20220309
@@ -89,6 +92,8 @@ class Sumstats():
         self.meta["Study"]=study
         self.log = Log()
         
+        _show_version(self.log)
+
         #preformat the data
         self.data  = preformat(
           sumstats=sumstats,
@@ -150,18 +155,21 @@ class Sumstats():
                     fixallele_args={},
                     sanitycheckstats_args={},
                     normalize=True,
-                    normalizeallele_args={}):
+                    normalizeallele_args={},
+                    verbose=True):
         ###############################################
         # try to fix data without dropping any information
-        self.data = fixID(self.data,**fixid_args)
+        self.data = fixID(self.data,verbose=verbose, **fixid_args)
         if remove is True:
-            self.data = removedup(self.data,log=self.log,**removedup_args)
-        self.data = fixchr(self.data,log=self.log,remove=remove,**fixchr_agrs)
-        self.data = fixpos(self.data,log=self.log,remove=remove,**fixpos_args)
-        self.data = fixallele(self.data,log=self.log,**fixallele_args)
-        self.data = sanitycheckstats(self.data,log=self.log,**sanitycheckstats_args)
+            self.data = removedup(self.data,log=self.log,verbose=verbose,**removedup_args)
+        self.data = fixchr(self.data,log=self.log,remove=remove,verbose=verbose,**fixchr_agrs)
+        self.data = fixpos(self.data,log=self.log,remove=remove,verbose=verbose,**fixpos_args)
+        self.data = fixallele(self.data,log=self.log,verbose=verbose,**fixallele_args)
+        self.data = sanitycheckstats(self.data,log=self.log,verbose=verbose,**sanitycheckstats_args)
         if normalize is True:
-            self.data = parallelnormalizeallele(self.data,n_cores=n_cores,log=self.log,**normalizeallele_args)
+            self.data = parallelnormalizeallele(self.data,n_cores=n_cores,verbose=verbose,log=self.log,**normalizeallele_args)
+        self.data = sortcoordinate(self.data,verbose=verbose,log=self.log)
+        self.data = sortcolumn(self.data,verbose=verbose,log=self.log)
         ###############################################
         
     
@@ -411,6 +419,34 @@ class Sumstats():
                            **args)
         # return sumstats object    
         return output
+
+    def get_density(self, sig_list=None, windowsizekb=100,
+                    large_number=10000000000,**args):
+        
+        if "SNPID" in self.data.columns:
+            id_to_use = "SNPID"
+        else:
+            id_to_use = "rsID"
+        
+        if sig_list is None:
+            self.data["DENSITY"] = getsignaldensity(self.data,
+                                                    id=id_to_use,
+                                                    chrom="CHR",
+                                                    pos="POS",
+                                                    bwindowsizekb=windowsizekb,
+                                                    large_number=large_number,
+                                                    log=self.log)
+        else:
+            if isinstance(sig_list, pd.DataFrame):
+                self.data["DENSITY"] = assigndensity(self.data,
+                                                    sig_list,
+                                                    id=id_to_use, 
+                                                    chrom="CHR", 
+                                                    pos="POS", 
+                                                    bwindowsizekb=windowsizekb,
+                                                    log=self.log)
+
+        
     def get_novel(self, **args):
         if "SNPID" in self.data.columns:
             id_to_use = "SNPID"
