@@ -34,6 +34,7 @@ def mqqplot(insumstats,
           snpid=None,
           eaf=None,
           chr_dict = get_chr_to_number(),
+          xtick_chr_dict = get_number_to_chr(),
           vcf_path=None,
           vcf_chr_dict = get_number_to_chr(),
           gtf_path="default",
@@ -66,6 +67,12 @@ def mqqplot(insumstats,
           mqqratio=3,
           bwindowsizekb = 100,
           large_number = 10000000000,
+          density_color=False,
+          density_range=(0,15),
+          density_trange=(0,10),
+          density_threshold=5,
+          density_tpalette="Blues",
+          density_palette="Reds",
           windowsizekb=500,
           anno=None,
           anno_set=[],
@@ -98,15 +105,17 @@ def mqqplot(insumstats,
           title =None,
           mtitle=None,
           qtitle=None,
-          figargs= {"figsize":(15,5)},
+          title_pad=1.08, 
+          title_fontsize=13,
           fontsize = 10,
+          anno_fontsize = 10,
+          figargs= {"figsize":(15,5)},
           colors=["#597FBD","#74BAD3"],
           marker_size=(5,25),
           use_rank=False,
           verbose=True,
           repel_force=0.03,
           build="19",
-          title_pad=1.08, 
           dpi=200,
           save=None,
           saveargs={"dpi":300,"facecolor":"white"},
@@ -224,6 +233,9 @@ def mqqplot(insumstats,
         else:
             raise ValueError("Please make sure "+anno+" column is in input sumstats.")
     
+    if (density_color==True) or ("b" in mode and "DENSITY" in insumstats.columns):
+        usecols.append("DENSITY")
+
     sumstats = insumstats.loc[:,usecols].copy()
 
 
@@ -306,7 +318,7 @@ def mqqplot(insumstats,
             sumstats.loc[right_chr&up_pos&low_pos,"HUE"]="0"
 
 # Density #####################################################################################################              
-    if "b" in mode:
+    if "b" in mode and "DENSITY" not in sumstats.columns:
         if verbose:log.write(" -Calculating DENSITY with windowsize of ",bwindowsizekb ," kb")
         stack=[]
         sumstats["TCHR+POS"] = sumstats[chrom]*large_number +  sumstats[pos]
@@ -323,6 +335,10 @@ def mqqplot(insumstats,
         sumstats["DENSITY"] = df["DENSITY"].values
         bmean=sumstats["DENSITY"].mean()
         bmedian=sumstats["DENSITY"].median()
+    elif "b" in mode and "DENSITY" in sumstats.columns:
+        bmean=sumstats["DENSITY"].mean()
+        bmedian=sumstats["DENSITY"].median()
+        if verbose:log.write(" -DENSITY column exists. Skipping calculation...")
      
     #############################
 # P value conversion #####################################################################################################  
@@ -457,7 +473,10 @@ def mqqplot(insumstats,
 ## Manhatann plot ###################################################
         if verbose:log.write("Start to create manhattan plot with "+str(len(sumstats))+" variants:")
         ## default seetings
+        
         palette = sns.color_palette(colors,n_colors=sumstats[chrom].nunique())  
+        
+
         legend = None
         style=None
         linewidth=0
@@ -494,15 +513,46 @@ def mqqplot(insumstats,
         
         ## if not highlight    
         else:
-            plot = sns.scatterplot(data=sumstats, x='i', y='scaled_P',
-                   hue='chr_hue',
-                   palette= palette,
+            if density_color == True:
+                hue = "DENSITY"
+                s = "DENSITY"
+                to_plot = sumstats.sort_values("DENSITY")
+
+                plot = sns.scatterplot(data=to_plot.loc[to_plot["DENSITY"]<=density_threshold], x='i', y='scaled_P',
+                       hue=hue,
+                       palette= density_tpalette,
+                       legend=legend,
+                       style=style,
+                       size=s,
+                       sizes=(marker_size[0]+1,marker_size[0]+1),
+                       linewidth=linewidth,
+                       hue_norm=density_trange,
+                       zorder=2,ax=ax1,edgecolor="black",**scatter_kwargs) 
+                plot = sns.scatterplot(data=to_plot.loc[to_plot["DENSITY"]>density_threshold], x='i', y='scaled_P',
+                   hue=hue,
+                   palette= density_palette,
                    legend=legend,
                    style=style,
-                   size="s",
+                   size=s,
                    sizes=marker_size,
+                   hue_norm=density_range,
                    linewidth=linewidth,
                    zorder=2,ax=ax1,edgecolor="black",**scatter_kwargs)   
+            else:
+                s = "s"
+                hue = 'chr_hue'
+                hue_norm=None
+                to_plot = sumstats
+                plot = sns.scatterplot(data=to_plot, x='i', y='scaled_P',
+                       hue=hue,
+                       palette= palette,
+                       legend=legend,
+                       style=style,
+                       size=s,
+                       sizes=marker_size,
+                       hue_norm=hue_norm,
+                       linewidth=linewidth,
+                       zorder=2,ax=ax1,edgecolor="black",**scatter_kwargs)   
 
         
         ## if pinpoint variants
@@ -656,7 +706,8 @@ def mqqplot(insumstats,
                            
         #plot.set_xlabel(chrom); 
         ax1.set_xticks(chrom_df.astype("float64"))
-        ax1.set_xticklabels(chrom_df.index.map(get_number_to_chr()),fontsize=fontsize,family="sans-serif")
+
+        ax1.set_xticklabels(chrom_df.index.astype("Int64").map(xtick_chr_dict),fontsize=fontsize,family="sans-serif")
         
         ## regional plot - set X tick
         if region is not None:
@@ -713,7 +764,7 @@ def mqqplot(insumstats,
                                sig_level=sig_level,
                                windowsizekb=windowsizekb,
                                verbose=False)
-                if to_annotate.empty is not True:
+                if (to_annotate.empty is not True) and ("b" not in mode):
                     if verbose: log.write(" -Found "+str(len(to_annotate))+" significant variants with a sliding window size of "+str(windowsizekb)+" kb...")
         else:
             to_annotate=getsig(sumstats.loc[sumstats["scaled_P"]> float(-np.log10(sig_level)),:],
@@ -724,7 +775,7 @@ def mqqplot(insumstats,
                                windowsizekb=windowsizekb,
                                verbose=False,
                                sig_level=sig_level)
-            if to_annotate.empty is not True:
+            if (to_annotate.empty is not True) and ("b" not in mode):
                 if verbose: log.write(" -Found "+str(len(to_annotate))+" significant variants with a sliding window size of "+str(windowsizekb)+" kb...")
         if (to_annotate.empty is not True) and anno=="GENENAME":
             to_annotate = annogene(to_annotate,
@@ -770,9 +821,9 @@ def mqqplot(insumstats,
         if verbose: log.write("Finished creating Manhattan plot successfully!")
         if mtitle and anno and len(to_annotate)>0: 
             pad=(ax1.transData.transform((skip, title_pad*maxy))[1]-ax1.transData.transform((skip, maxy)))[1]
-            ax1.set_title(mtitle,pad=pad,fontsize=fontsize,family="sans-serif")
+            ax1.set_title(mtitle,pad=pad,fontsize=title_fontsize,family="sans-serif")
         elif mtitle:
-            ax1.set_title(mtitle,fontsize=fontsize,family="sans-serif")
+            ax1.set_title(mtitle,fontsize=title_fontsize,family="sans-serif")
             
        
         if anno and (to_annotate.empty is not True):
@@ -875,7 +926,7 @@ def mqqplot(insumstats,
                 else:
                     bbox_para=None
                 
-                anno_default = {"rotation":40,"style":"italic","ha":"left","va":"bottom","fontsize":fontsize,"fontweight":fontweight}
+                anno_default = {"rotation":40,"style":"italic","ha":"left","va":"bottom","fontsize":anno_fontsize,"fontweight":fontweight}
                 for key,value in anno_args.items():
                     anno_default[key]=value
                    
@@ -980,9 +1031,10 @@ def mqqplot(insumstats,
                 ax2.set_yticks([x for x in range(skip,cut+1,2)])
                 ax2.set_yticklabels([x for x in range(skip,cut+1,2)],fontsize=fontsize,family="sans-serif")
         
+        ax2.tick_params(axis='both', which='both', labelsize=fontsize)
         #
         if qtitle:
-            ax2.set_title(qtitle,fontsize=fontsize,pad=10,family="sans-serif")
+            ax2.set_title(qtitle,fontsize=title_fontsize,pad=10,family="sans-serif")
 
         if verbose: log.write("Finished creating QQ plot successfully!")
 # Creating QQ plot Finished #############################################################################################
@@ -1001,9 +1053,9 @@ def mqqplot(insumstats,
     # add title 
     if title and anno and len(to_annotate)>0:
         # increase height if annotation 
-        fig.suptitle(title ,x=0.5, y=1.05)
+        fig.suptitle(title , fontsize = title_fontsize ,x=0.5, y=1.05)
     else:
-        fig.suptitle(title ,x=0.5,y=1)
+        fig.suptitle(title , fontsize = title_fontsize, x=0.5,y=1)
     
     garbage_collect.collect()
 # Return matplotlib figure object #######################################################################################
@@ -1072,18 +1124,31 @@ def process_vcf(sumstats, vcf_path, region, log, verbose, pos , region_ld_thresh
 
 def process_gtf(gtf_path,region,region_flank_factor,build,region_protein_coding,gtf_chr_dict,gtf_gene_name):
     #loading
+    
+    # chr to string datatype using gtf_chr_dict
+    to_query_chrom = gtf_chr_dict[region[0]]
+
+    # loading gtf data
     if gtf_path =="default" or gtf_path =="ensembl":
-        to_query_chrom = gtf_chr_dict[region[0]]
-        gtf = get_gtf(chrom=to_query_chrom,build=build,source="ensembl")
+        
+        gtf = get_gtf(chrom=to_query_chrom, build=build, source="ensembl")
+    
     elif gtf_path =="refseq":
-        gtf = get_gtf(chrom=to_query_chrom,build=build,source="refseq")
+
+        gtf = get_gtf(chrom=to_query_chrom, build=build, source="refseq")
+    
     else:
-        gtf = pd.read_csv(gtf_path,sep="\t",header=None, comment="#",low_memory=False,dtype={0:"string"})
+        # if user-provided gtf
+        gtf = pd.read_csv(gtf_path,sep="\t",header=None, comment="#", low_memory=False,dtype={0:"string"})
         gtf = gtf.loc[gtf[0]==gtf_chr_dict[region[0]],:]
-    #filter in region
-    #gtf_chr_dict
+
+    # filter in region
     genes_1mb = gtf.loc[(gtf[0]==to_query_chrom)&(gtf[3]<region[2])&(gtf[4]>region[1]),:].copy()
+    
+    # extract biotype
     genes_1mb.loc[:,"gene_biotype"] = genes_1mb[8].str.extract(r'gene_biotype "([\w\.\_-]+)"')
+    
+    # extract gene name
     if gtf_gene_name is None:
         if gtf_path=="refseq":
             genes_1mb.loc[:,"name"] = genes_1mb[8].str.extract(r'gene_id "([\w\.-]+)"').astype("string")
@@ -1095,22 +1160,46 @@ def process_gtf(gtf_path,region,region_flank_factor,build,region_protein_coding,
         pattern = r'{} "([\w\.-]+)"'.format(gtf_gene_name)
         genes_1mb.loc[:,"name"] = genes_1mb[8].str.extract(pattern).astype("string")
 
+    # extract gene
     genes_1mb.loc[:,"gene"] = genes_1mb[8].str.extract(r'gene_id "([\w\.-]+)"')
-
+    
+    # extract exon
     exons = genes_1mb.loc[genes_1mb[2]=="exon",:].copy()
+
+    # extract protein coding gene
     if region_protein_coding is True:
         genes_1mb  =  genes_1mb.loc[genes_1mb["gene_biotype"]=="protein_coding",:].copy()
+    
     #uniq genes
+    ## get all record with 2nd column == gene
     uniq_gene_region = genes_1mb.loc[genes_1mb[2]=="gene",:].copy()
+    
+    ## extract region + flank
     flank = region_flank_factor * (region[2] - region[1])
+    
+    ## get left and right boundary
     uniq_gene_region["left"] = uniq_gene_region[3]-flank
     uniq_gene_region["right"] = uniq_gene_region[4]+flank
 
-    ## arrange rows
-    stacks=[]
-    stack_dic={}
-    for index,row in uniq_gene_region.sort_values([3]).iterrows():
+    # arrange gene track
+    stack_dic = assign_stack(uniq_gene_region.sort_values([3]).loc[:,["name","left","right"]])  
+
+    # map gene to stack and add stack column : minus stack
+    uniq_gene_region["stack"] = -uniq_gene_region["name"].map(stack_dic)
+    exons.loc[:,"stack"] = -exons.loc[:,"name"].map(stack_dic)
+
+    # return uniq_gene_region (gene records with left and right boundary)
+    # return exon records with stack number
+    return uniq_gene_region, exons
+
+def assign_stack(uniq_gene_region):
+
+    stacks=[] ## stack : gene track
+    stack_dic={} # mapping gene name to stack
+    
+    for index,row in uniq_gene_region.iterrows():
         if len(stacks)==0:
+            # add first entry 
             stacks.append([(row["left"],row["right"])])
             stack_dic[row["name"]] = 0
         else:
@@ -1142,10 +1231,7 @@ def process_gtf(gtf_path,region,region_flank_factor,build,region_protein_coding,
                             break
                 if row["name"] in stack_dic.keys():
                     break         
-    uniq_gene_region["stack"] = -uniq_gene_region["name"].map(stack_dic)
-    exons.loc[:,"stack"] = -exons.loc[:,"name"].map(stack_dic)
-    return uniq_gene_region, exons
-
+    return stack_dic
 
 def closest_gene(x,data,chrom="CHR",pos="POS",maxiter=20000,step=50):
         gene = data.gene_names_at_locus(contig=x[chrom], position=x[pos])
